/*
 * [codejive-common] Codejive commons package
 * 
 * Copyright (C) 2006 Tako Schotanus
 * 
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA.
 * 
 * Linking this library statically or dynamically with other modules is
 * making a combined work based on this library.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on this library.  If you modify this library, you may extend
 * this exception to your version of the library, but you are not
 * obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 * 
 * Created on Apr 14, 2005
 */
package org.codejive.common.cache;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

public class HashedObjectCache<K, T> implements ObjectCache<K, T> {
	private HashMap<K, CachedItem> items;
	
	private class CachedItem {
		public K key;
		public T object;
		public UpdateMonitor monitor;
		
		public CachedItem(K _key, T _object, UpdateMonitor _monitor) {
			key = _key;
			object = _object;
			monitor = _monitor;
		}
	}

	public HashedObjectCache() {
		items = new HashMap<K, CachedItem>();
	}
	
	public T put(K _key, T _value) {
		return put(_key, _value, null);
	}
	
	public T put(K _key, T _value, UpdateMonitor _monitor) {
		CachedItem item = new CachedItem(_key, _value, _monitor);
		items.put(_key, item);
		return _value;
	}

	public void putAll(Map<? extends K, ? extends T> t) {
		// TODO Auto-generated method stub
		
	}

	public int size() {
		return items.size();
	}

	public boolean isEmpty() {
		return items.isEmpty();
	}

	public boolean containsKey(Object _key) {
		return items.containsKey(_key);
	}

	public boolean containsValue(Object _value) {
		return items.containsValue(_value);
	}

	public T get(Object _key) {
		T value = null;
		CachedItem item = items.get(_key);
		if (item != null) {
			if ((item.monitor == null) || (!item.monitor.isUpdated())) {
				value = item.object;
			} else {
				items.remove(_key);
			}
		}
		return value;
	}

	public T remove(Object _key) {
		T value = null;
		CachedItem item = items.remove(_key);
		if (item != null) {
			value = item.object;
		}
		return value;
	}

	public void clear() {
		items.clear();
	}

	public Set<K> keySet() {
		return items.keySet();
	}

	public Collection<T> values() {
		// TODO Auto-generated method stub
		return null;
	}

	public Set<Entry<K, T>> entrySet() {
		// TODO Auto-generated method stub
		return null;
	}
}


/*
 * $Log:	$
 */